import roof_draw.templates as templates
import roof_draw.utils as utils
from qgis.PyQt.QtWidgets import QFileDialog
import tempfile

try:
    from osgeo import gdal
except ImportError:
    import gdal

import os.path
import os

import shutil
from qgis.core import QgsProject, QgsRectangle, QgsReferencedRectangle
from datetime import date

""" Inicializa o form 2 vetorizar telhados imagens e salvar no banco """
def initialize_roof_draw():
    # file/folder chooser init
    utils.DLG_DRAW_ROOFS.file_train.clear()
    utils.DLG_DRAW_ROOFS.btn_file_train.clicked.connect(select_input_file)
    populate_select_area()
    utils.populate_select_layers_shp(utils.DLG_DRAW_ROOFS.comboBox_camada_exclusao)

def run_roof_draw():
        """Run method that performs all the real work"""
        # Create the dialog with elements (after translation) and keep reference
        # Only create GUI ONCE in callback, so that it will only load when the plugin is started

        project = QgsProject.instance()
        root = project.layerTreeRoot()
        layer = utils.iface.activeLayer()
        
        if(layer.name() == "Bing Satellite" and root.findLayer(layer.id()).isVisible() and len(root.checkedLayers())==1):
            if utils.first_start_draw_roofs == True:
                utils.first_start_draw_roofs = False
                utils.DLG_DRAW_ROOFS = templates.RoofDrawDrawRoofsDialog()
                initialize_roof_draw()

            # show the dialog
            utils.DLG_DRAW_ROOFS.show()
            # Run the dialog event loop
            result = utils.DLG_DRAW_ROOFS.exec_()
            if result:
                ok_to_roof_draw()
        else:
            utils.iface.messageBar().pushMessage("ERRO", "Abra a camada Bing Satellite e a selecione-a, SOMENTE ela deve estar VISIVEL!", level=1)

"""acao botao ok"""       
def ok_to_roof_draw():
    utils.iface.messageBar().pushMessage("INFO", "Processo Iniciado Por Favor Aguarde !...", level=0)
    train_path = utils.DLG_DRAW_ROOFS.file_train.text()

    area_from = utils.DLG_DRAW_ROOFS.comboBox_quadrante_inicial.currentText()
    area_to = utils.DLG_DRAW_ROOFS.comboBox_quadrante_final.currentText()
    camada_exclusao = utils.DLG_DRAW_ROOFS.comboBox_camada_exclusao.currentText()
    area_id = utils.DLG_DRAW_ROOFS.comboBox_area.currentIndex()

    if (train_path  != ""
        and area_from != ""
        and area_to != ""):
        date_created = date.today()
        area_from = int(area_from)
        area_to = int(area_to)
        if(area_from <= area_to):
            for id_pk in range(area_from, area_to):

                # TODO extrair essa lógica executar primeiro o image cut e colocar isso em seguida ao corte de cada imagem
                result = utils.area_bbox(utils.CONNECTION, id_pk)
                not_allowed_areas_layer = None
                if(len(QgsProject.instance().mapLayersByName(camada_exclusao))>0):
                    not_allowed_areas_layer = QgsProject.instance().mapLayersByName(camada_exclusao)[0]
                images_path =  tempfile.mkdtemp(prefix="roofdraw_img",suffix="_tmp")
                generate_images(images_path, result[0][0], result[0][1], result[0][2], result[0][3], not_allowed_areas_layer, id_pk)
                draw_roofs(images_path, train_path,  id_pk, date_created)
                utils.iface.messageBar().pushMessage("INFO", "Roof Draw - finalizou para o quadrante "+str(id_pk), level=0)
                utils.DLG_DRAW_ROOFS.destroy()
            # E CRIADA UM VIEW QUE FAZ UM UNION NOS POLIGONOS
        else:
            utils.iface.messageBar().pushMessage("ERRO", "Quadrante final deve ser maior ou igual a quadrante inicial", level=1)
    else:
        utils.iface.messageBar().pushMessage("ERRO", "Image Cut - preencher formulário corretamente", level=1)
        utils.DLG_DRAW_ROOFS.destroy()

'''percorre o raster camada base e salva pedacinhos dela como png'''
def generate_images(images_path, limit_x_min, limit_y_min, limit_x_max, limit_y_max, not_allowed_areas_layer, area_id):
    print(limit_x_min, limit_y_min, limit_x_max, limit_y_max)
    # Inicializa o caminhar pela camada basica
    next_y_min = limit_y_min
    next_x_min = limit_x_min
    map_update_extent(limit_x_min,limit_y_min,next_x_min,next_y_min)
    e = utils.iface.mapCanvas().extent()
    xmax = e.xMaximum()
    ymax =  e.yMaximum()
    xmin = e.xMinimum()
    ymin = e.yMinimum()
    count_images = 0
    # salvar primeira imagem
    count_images = export_image(xmin,ymin,xmax,ymax,not_allowed_areas_layer, count_images, area_id, images_path)
    
    while(ymax < limit_y_max):   
        while(xmax < limit_x_max):
            print("entra..")
            e = utils.iface.mapCanvas().extent()
            xmin = xmax
            xmax = xmax + utils.STEP_X
            map_update_extent(xmin,ymin,xmax,ymax)
            count_images = export_image(xmin,ymin,xmax,ymax,not_allowed_areas_layer, count_images, area_id, images_path)
        # resetar o x
        ymin = ymin + utils.STEP_Y
        ymax =  ymax + utils.STEP_Y
        xmin = limit_x_min
        xmax = limit_x_min + utils.STEP_X

def export_image(xmin,ymin,xmax,ymax,not_allowed_areas_layer, count_images, area_id, images_path):
    is_valid = utils.is_valid_area(xmin,ymin,xmax,ymax,not_allowed_areas_layer)
    if utils.is_valid_area(xmin,ymin,xmax,ymax,not_allowed_areas_layer):
        imgname = f'area_{area_id}_img_{count_images}'
        filePath = f'{images_path}/{imgname}.png'
        # salva imagem no local
        utils.iface.mapCanvas().saveAsImage(filePath)
        count_images += 1
    return count_images
    
""" Atualiza o extent do mapa do bing que o usuario abriu no QGIS """
def map_update_extent( xmin,ymin,xmax,ymax):
    region = QgsRectangle(xmin,ymin,xmax,ymax)
    region_referenced = QgsReferencedRectangle(region, utils.CRS)
    utils.iface.mapCanvas().setExtent(region_referenced)
    utils.iface.mapCanvas().zoomScale(543)
    utils.iface.mapCanvas().refresh()
    utils.iface.mapCanvas().waitWhileRendering()

'''selecionar arquivo'''
def select_input_file():
    filename = QFileDialog.getOpenFileName(utils.DLG_DRAW_ROOFS, "Seleciona o arquivo modelo","", "*.onnx")
    utils.DLG_DRAW_ROOFS.file_train.setText(filename[0])

'''selecionar pasta'''
def select_output_input_folder():
    filename = QFileDialog.getExistingDirectory(utils.DLG_DRAW_ROOFS, "Seleciona a pasta de imagens")
    utils.DLG_DRAW_ROOFS.dir_images.setText(filename)

def populate_select_area():
    utils.DLG_DRAW_ROOFS.comboBox_area.clear()
    result = utils.list_description(utils.CONNECTION)
    utils.DLG_DRAW_ROOFS.comboBox_area.addItem("")
    for area in result:
        utils.DLG_DRAW_ROOFS.comboBox_area.addItem(area[1], area[0])
    utils.DLG_DRAW_ROOFS.comboBox_area.currentIndexChanged.connect(populate_quadrante)

def populate_quadrante():
    utils.DLG_DRAW_ROOFS.comboBox_quadrante_inicial.clear()
    utils.DLG_DRAW_ROOFS.comboBox_quadrante_final.clear()
    area_id = utils.DLG_DRAW_ROOFS.comboBox_area.currentIndex()
    result = utils.list_id_description(utils.CONNECTION, area_id)
    list_ids = [str(id_pk[0]) for id_pk in result]
    utils.DLG_DRAW_ROOFS.comboBox_quadrante_inicial.addItems(list_ids)
    utils.DLG_DRAW_ROOFS.comboBox_quadrante_final.addItems(list_ids)

"""Vetoriza"""
def images_to_vetor( path_images, onnx_file,  area_id, date_created):
    temp_path = tempfile.mkdtemp(prefix="roofdraw_",suffix="_tmp")
    infer_worker = utils.InferWorker(onnx_file)
    filelist=os.listdir(path_images)
    filelist = [fichier for fichier in filelist if fichier.endswith(".png")]
    for fichier in filelist:
            
        complete_img_path = path_images+"/"+fichier
        layer_path = utils.dowm_sample(complete_img_path, temp_path, utils.SCALE)
        
        band_list = [1, 2, 3]
        
        ras_ds = gdal.Open(layer_path)
        geot = ras_ds.GetGeoTransform()
        
        ras_ds.SetProjection(utils.PROJ.ExportToWkt())
        
        proj = ras_ds.GetProjection()
        
        # If this layer is a raster layer
        xsize = ras_ds.RasterXSize
        ysize = ras_ds.RasterYSize
        ras_ds = None
        grid_count = utils.create_grids(ysize, xsize, utils.BLOCKSIZE, utils.OVERLAPSIZE)
        
        number = grid_count[0] * grid_count[1]
        
        geoinfo = {"row": ysize, "col": xsize, "geot": geot, "proj": proj}
        create_file_shp_name = temp_path+"/"+fichier.replace(".png", ".shp")
        
        mask_save_path = create_file_shp_name.replace(".shp", "_down.tif")
        
        mask = utils.Mask(mask_save_path, geoinfo, utils.BLOCKSIZE, utils.OVERLAPSIZE)
        for i in range(grid_count[0]):
            
            for j in range(grid_count[1]):
                
                img = utils.layer2array(layer_path, band_list, \
                                        i, j, utils.BLOCKSIZE, utils.OVERLAPSIZE)
            
                infer_img = infer_worker.infer(img, True)
        
                try:
                    mask.write_grid(infer_img, i, j)
                except:
                    utils.iface.messageBar().pushMessage("INFO", "Roof Draw - pode ter encontrado uma área vazia sem telhados: "+complete_img_path, level=0)
        utils.polygonize_raster(utils.CONNECTION, mask, create_file_shp_name, proj, geot,  area_id, \
                                        display=(not utils.SIMPLYFY_POLYGON))
    
    # REMOVE TMP IMG DIR
    shutil.rmtree(path_images)
    utils.iface.messageBar().pushMessage("INFO", f"Foram poligonizadas {len(filelist)} imagens", level=0)

""" Vetoriza telhados de imagens tif e salvar no banco """
def draw_roofs(path_images, onnx_file,  area_id, date_created):
    images_to_vetor(path_images, onnx_file,  area_id, date_created)
    update_quadrante(area_id)
    utils.iface.messageBar().pushMessage("INFO", "Gerando arquivo de resultado...", level=0)

# update quadrante flag
def update_quadrante(id_pk):
    conn = utils.CONNECTION
    conn.executeSql("UPDATE roof_drawing_quadrante SET flag = true WHERE id_pk = "+str(id_pk))