import os
import os.path as osp
import numpy as np
from qgis.utils import iface
import psycopg2
from qgis.core import QgsVectorLayer, QgsDataSourceUri, QgsProject, QgsCoordinateReferenceSystem, QgsRasterLayer, QgsRectangle, QgsPointXY, QgsGeometry, QgsFeature, QgsFeatureRequest, QgsMapLayerType
import processing
import base64
import numpy as np
from .progressbar import create_progressbar, clear_messagebar
from .defaults import CRS_id


try:
    from osgeo import gdal, ogr, osr
except ImportError:
    import gdal
    import ogr
    import osr

PATH = os.path.realpath(__file__)
print(PATH)

def __mask2tif(mask, tmp_path, proj, geot):
    row, columns = mask.shape[:2]
    driver = gdal.GetDriverByName("GTiff")
    dst_ds = driver.Create(tmp_path, columns, row, 1, gdal.GDT_UInt16)
    dst_ds.SetGeoTransform(geot)
    dst_ds.SetProjection(proj)
    dst_ds.GetRasterBand(1).WriteArray(mask)
    dst_ds.FlushCache()
    return dst_ds

# "5432","10.10.0.3", "plugin_db", "postgres", "postgres", shp_save_path, ds, srcband, maskband, "PostgreSQL"
def polygonize_raster(conn, mask, shp_save_path, proj, geot,  area_id, rm_tmp=True, display=True):
    if type(mask) is np.ndarray:
        tmp_path = shp_save_path.replace(".shp", ".tif")
        ds = __mask2tif(mask, tmp_path, proj, geot)
    else:
        tmp_path = mask.file_name
        ds = mask.gdal_data
    srcband = ds.GetRasterBand(1)
    maskband = srcband.GetMaskBand()
    gdal.SetConfigOption("GDAL_FILENAME_IS_UTF8", "YES")
    gdal.SetConfigOption("SHAPE_ENCODING", "UTF-8")
    ogr.RegisterAll()
    config = save_to_db(conn, shp_save_path, ds, srcband, maskband, rm_tmp, mask, area_id, "PostgreSQL")

# Lembrar de configurar o db para acesso externo
#  vim /etc/postgresql/13/main/pg_hba.conf
#  service postgresql restart
# create extension postgis;
# select ST_AsText(wkb_geometry::geometry) from roof_drawing limit 1;
def save_to_db(conn, shp_save_path, ds, srcband, maskband, rm_tmp, mask, area_id, driver_name="PostgreSQL"):
    if driver_name == "PostgreSQL":
        driver = ogr.GetDriverByName("Memory")
        dataSource =  driver.CreateDataSource(shp_save_path)
        id_index_start = shp_save_path.split("/")[-1].replace(".shp","")
       
        prosrs = osr.SpatialReference(wkt=ds.GetProjection())
        layer = dataSource.CreateLayer("Building boundary", geom_type=ogr.wkbPolygon, srs=prosrs)
        field_definition_dn = ogr.FieldDefn("DN", ogr.OFTInteger)
        layer.CreateField(field_definition_dn)
        
        gdal.Polygonize(srcband, maskband, layer, 0, [])

        layer = dataSource.GetLayer()
        layer.SetAttributeFilter("DN != '0'")

        save_each_feature(conn, layer,  id_index_start, area_id)

        return 0  
    else:
        return 1

def save_each_feature(conn, layer,  description_text, area_id):
    for feature in layer:
        if feature.GetGeometryRef().Clone() is not None and feature.GetGeometryRef().Clone() !="":
            wkt = feature.GetGeometryRef().ExportToWkt()
            dn = feature.GetFieldAsString("DN")
            insert_into_roof_drawing(conn, wkt, dn, description_text, area_id)

def save_each_feature_QgsVectorLayer(conn, layer,  area_id):
    for feature in layer.getFeatures():
        idx_dn = feature.fieldNameIndex('DN')
        dn = str(feature.attributes()[idx_dn])
        wkt = feature.geometry().asWkt()
        geom = wkt
        insert_into_roof_drawing_QgsVectorLayer(conn, geom, dn, area_id)

"""Salvar área de avaliacao da geom"""
# QgsAbstractDatabaseProviderConnection
def insert_into_roof_drawing_quadrante_from_wkt(conn, geom, area_id):
    result = conn.executeSql(f"INSERT INTO roof_drawing_quadrante (id_pk, the_geom, area_id, flag) VALUES(nextval('roof_drawing_area_id_seq'), ST_GeomFromText('{geom}',3857), {area_id}, false)")
    return recover_last_id(conn, 'roof_drawing_quadrante')

def insert_into_roof_drawing_area(conn, description_text):
    result = conn.executeSql(f"INSERT INTO roof_drawing_area (id_pk, nome, flag) VALUES(nextval('roof_drawing_area_id_seq'),BTRIM('{description_text}'), false)")
    return recover_last_id(conn, 'roof_drawing_area')

"""Salvar multiplas áreas receber um poligono criar um grid de 1000 x 1000  
    recortar o grid"""
# QgsAbstractDatabaseProviderConnection
def insert_into_roof_drawing_area_multiple(conn, layer, description_text, hspacing=1000, vspacing=1000):
    ex = layer.extent()
    bbox = str(ex.xMinimum()) + ','+str(ex.xMaximum())+','+str(ex.yMinimum())+","+str(ex.yMaximum())
    feedback = create_progressbar()
    grid = processing.run("native:creategrid", {'TYPE':2, 'EXTENT':bbox, 'HSPACING': hspacing, 'VSPACING': vspacing, 'CRS':'EPSG:3857','OUTPUT':'memory:'}, feedback=feedback)['OUTPUT']
    grid_cut = processing.run("native:clip",  {'INPUT':grid, 'OVERLAY':layer,'OUTPUT':'memory:'}, feedback=feedback)['OUTPUT']
    area_id = insert_into_roof_drawing_area(conn, description_text)
    count_features = 0
    for feature in grid_cut.getFeatures():
        print(type(feature))
        # feature.geometry() - multi polygon
        for geometry in feature.geometry().asGeometryCollection():
            count_features += 1
            wkt_geometry = geometry.asWkt()
            insert_into_roof_drawing_quadrante_from_wkt(conn, wkt_geometry, area_id)
    clear_messagebar()

"""recupera último id"""
def recover_last_id(connector, table_name):
    result = connector.executeSql(f"SELECT MAX(id_pk) FROM {table_name}")
    current_id = None
    for row in result:
        current_id = row[0]
    return current_id

"""lista as distintas descricoes de area"""
def list_description(conn):
    result = conn.executeSql(f"SELECT id_pk, nome FROM roof_drawing_area")
    return result

"""lista os ids de quadrante onde area igual ..."""
def list_id_description(conn, area_id):
    result = conn.executeSql(f"SELECT id_pk FROM roof_drawing_quadrante WHERE area_id = '{area_id}'")
    return result

"""get bbox"""
def area_bbox(conn, id_pk):
    result = conn.executeSql(f"SELECT ST_XMin(the_geom), ST_YMin(the_geom), ST_XMax(the_geom), ST_YMax(the_geom) FROM roof_drawing_quadrante WHERE id_pk = {id_pk}")
    return result

# QgsAbstractDatabaseProviderConnection
def insert_into_roof_drawing_QgsVectorLayer(conn, wkt_geometry, dn, area_id):
    area_id = str(area_id)
    conn.executeSql(f"INSERT INTO roof_drawing_fixed_dissolved_simplified (id_pk, the_geom, dn, area_id, flag) VALUES(nextval('roof_drawing_fixed_dissolved_simplified_id_seq'), ST_GeomFromText('{wkt_geometry}',3857), {dn},{area_id}, false)")

# QgsAbstractDatabaseProviderConnection
def insert_into_roof_drawing(conn, wkt_geometry, dn, description, quadrante_id):
    quadrante_id = str(quadrante_id)
    conn.executeSql(f"INSERT INTO roof_drawing (id_pk, the_geom, dn, description_text, quadrante_id, flag) VALUES(nextval('roof_drawing_id_seq'), ST_GeomFromText('{wkt_geometry}',3857), {dn}, '{description}',{quadrante_id}, false)")

# QgsAbstractDatabaseProviderConnection
def file_init_sql(conn):
    file_sql = open(PATH.replace("shape.py","sql/init.sql"), "r")
    sql = file_sql.read()
    conn.executeSql(sql)
    file_sql.close()

def recover_db_layer(conn, table_name):
    uri = uri_from_connect_to_specific_table(conn, table_name)
    vlayer = QgsVectorLayer(uri, table_name, "postgres")
    return vlayer

def uri_from_connect_to_specific_table(conn, table_name):
    uri = QgsDataSourceUri(conn.uri())
    # schema, table, geometry, column, where filter, pk
    uri.setDataSource("public", table_name, "the_geom", "", "id_pk")
    return uri.uri()

# NAO È MAIS USADA - USAR EM FORM SEPARADO EXECUTA PARA A LAYER TODA
def fix_dissove_simplify(conn, area_description_text):
        # IMPORTANTE MANTER : EM memory: postgres:
        # recuperando camada do banco
        db_layer = recover_db_layer(conn, "roof_drawing")
        print(db_layer)
        feedback = create_progressbar()
        # fixing before insert DB
        fix_layer = processing.run("native:fixgeometries", {'INPUT':db_layer,'OUTPUT':'memory:'}, feedback=feedback)['OUTPUT']
        print(fix_layer)
        dissolved_result = processing.run("native:dissolve", {'INPUT':fix_layer, 'FIELD':['dn'], 'OUTPUT':'memory:'}, feedback=feedback)['OUTPUT']
        print(dissolved_result)
        uri = uri_from_connect_to_specific_table(conn, "roof_drawing_fixed_dissolved_simplified")
        print(uri)
        # sigle parts
        single_parts = processing.run("native:multiparttosingleparts", {'INPUT':dissolved_result, 'OUTPUT':'memory:'}, feedback=feedback)['OUTPUT']
        # recuperando camada do banco
        db_layer = recover_db_layer(conn, "roof_drawing")
        # adicionando a layer do banco na tela
        # QgsProject.instance().addMapLayer(db_layer)
        # Salvando depois no banco
        save_each_feature_QgsVectorLayer(conn, single_parts,  area_description_text)
        # db_layer_simple = recover_db_layer(conn, "roof_drawing_fixed_dissolved_simplified")
        # QgsProject.instance().addMapLayer(db_layer_simple)
        clear_messagebar()


"""retorna falso, ou seja, não é uma área válida se intersectar 
    com algum retangulo de área não permitida"""
def  is_valid_area(xmin,ymin,xmax,ymax, not_allowed_areas_layer):
    current_cut_rectangle = QgsRectangle(QgsPointXY(xmin,ymin), QgsPointXY(xmax, ymax))
    is_allowed = True
    if(not_allowed_areas_layer!=None):
        for feat in not_allowed_areas_layer.getFeatures(QgsFeatureRequest().setFilterRect(current_cut_rectangle)):
            is_allowed = False
            return is_allowed
    return is_allowed

"""verifica se layer esta na projecao correta"""
def verify_projection(layer, CRS_id):
    if layer.crs().authid() == CRS_id:
        return True
    else:
        return False

def file_to_bytea(path):
    with open(path, "rb") as current_file:
        f = current_file.read()
        binary = psycopg2.Binary(f)
        return binary

def populate_select_layers_shp(combobox):
    # limpar combobox antes
    combobox.clear()
    layers = [layer for layer in QgsProject.instance().mapLayers().values() if layer.type() == QgsMapLayerType(0)]
    layer_list = [""]
    for layer in layers:
        # tudo tem que trabalhar com a mesma projeção
        if verify_projection(layer, CRS_id):
            layer_list.append(layer.name())
            combobox.clear()
            combobox.addItems(layer_list)