CREATE SEQUENCE IF NOT EXISTS roof_drawing_area_id_seq;
CREATE TABLE IF NOT EXISTS roof_drawing_area(id_pk integer NOT NULL DEFAULT nextval('roof_drawing_area_id_seq'), nome varchar(25), flag boolean, PRIMARY KEY(id_pk));

CREATE SEQUENCE IF NOT EXISTS roof_drawing_quadrante_id_seq;
CREATE TABLE IF NOT EXISTS roof_drawing_quadrante(id_pk integer NOT NULL DEFAULT nextval('roof_drawing_quadrante_id_seq'), the_geom GEOMETRY, area_id integer, flag boolean, PRIMARY KEY(id_pk), CONSTRAINT fk_area FOREIGN KEY(area_id) REFERENCES roof_drawing_area(id_pk));
ALTER TABLE roof_drawing_quadrante ALTER COLUMN the_geom TYPE geometry(POLYGON, 3857);

CREATE SEQUENCE IF NOT EXISTS roof_drawing_id_seq;
CREATE TABLE IF NOT EXISTS roof_drawing(id_pk integer NOT NULL DEFAULT nextval('roof_drawing_id_seq'), dn integer, roof_type varchar(25), the_geom GEOMETRY, description_text varchar(25), quadrante_id integer, flag boolean, PRIMARY KEY(id_pk), CONSTRAINT fk_quadrante FOREIGN KEY(quadrante_id) REFERENCES roof_drawing_quadrante(id_pk));
ALTER TABLE roof_drawing ALTER COLUMN the_geom TYPE geometry(POLYGON, 3857);

-- CREATE SEQUENCE IF NOT EXISTS roof_drawing_fixed_dissolved_simplified_id_seq;
-- CREATE TABLE IF NOT EXISTS roof_drawing_fixed_dissolved_simplified(id_pk integer NOT NULL DEFAULT nextval('roof_drawing_fixed_dissolved_simplified_id_seq'), dn integer, roof_type varchar(25), the_geom GEOMETRY, flag boolean, description_text varchar(25), area_id integer, PRIMARY KEY(id_pk), CONSTRAINT fk_area FOREIGN KEY(area_id) REFERENCES roof_drawing_area(id_pk));
-- ALTER TABLE roof_drawing_fixed_dissolved_simplified ALTER COLUMN the_geom TYPE geometry(POLYGON, 3857);


-- FIX GEOMETRY
-- CRIA TABELA COM FEATURES UNIDAS QUANDO PERTO
CREATE TABLE IF NOT EXISTS roof_drawing_fix AS 
    SELECT 
    ROW_NUMBER() OVER (ORDER BY c.quadrante_id) as id_pk, 
    c.quadrante_id, c.roof_type, c.flag, c.dn, c.the_geom 
    FROM (SELECT (ST_Dump(ST_Union(d.the_geom)::geometry)).geom as the_geom,
          ROW_NUMBER() OVER (ORDER BY d.quadrante_id) as id_pk, 
          d.quadrante_id, d.roof_type, d.flag, d.dn
          FROM roof_drawing d
          GROUP BY d.quadrante_id, d.roof_type, d.flag, d.dn
         ) as c;
-- FUNCAO PARA ATUALIZAR TABELA 
CREATE OR REPLACE FUNCTION update_roof_drawing_fix() RETURNS trigger AS $emp_stamp$
    BEGIN
       DROP TABLE roof_drawing_fix;
       CREATE TABLE roof_drawing_fix AS 
       SELECT 
            ROW_NUMBER() OVER (ORDER BY c.quadrante_id) as id_pk, 
            c.quadrante_id, c.roof_type, c.flag, c.dn, c.the_geom 
            FROM (SELECT (ST_Dump(ST_Union(d.the_geom)::geometry)).geom as the_geom,
                  ROW_NUMBER() OVER (ORDER BY d.quadrante_id) as id_pk, 
                  d.quadrante_id, d.roof_type, d.flag, d.dn
                  FROM roof_drawing d
                  GROUP BY d.quadrante_id, d.roof_type, d.flag, d.dn
                 ) as c;
        RETURN NULL;
    END;
$emp_stamp$ LANGUAGE plpgsql;

--TRIGGER PARA QUANDO INSERIR EM roof_drawing ATUALIZAR roof_drawing_fix
DROP TRIGGER IF EXISTS trg_insert_roof_drawing_fix ON roof_drawing;
CREATE TRIGGER trg_insert_roof_drawing_fix AFTER INSERT ON roof_drawing FOR EACH ROW EXECUTE PROCEDURE update_roof_drawing_fix();
END;

